//PassFile.cpp

#include "PassFile.h"
#include "sha2.h"

using namespace std;

PassFile::PassFile(sha2::SHA_TYPE shaType, const char *fileName, const char *salt){
    fstream file;

    m_ShaType = shaType;
    m_strFileName = fileName;
    m_strSalt = salt;
    m_boolChangesMade = false;

    file.open(m_strFileName.c_str(), fstream::in);
    if (!file.is_open()){//it is fine if the file is missing
        return;
    }
    string user, cryptpass;
    while (!file.eof()){//no checking here for corrupted input file
        file >> user >> cryptpass;
        if (user.length() > 1)
            m_map[user] = cryptpass;
    }
    file.close();
}

PassFile::~PassFile(){
    flush2file();
}

void PassFile::flush2file(){
    fstream file;

    if (!m_boolChangesMade) return;

    file.open(m_strFileName.c_str(), fstream::out);
    if (!file){
        string strErr = "Can't open " + m_strFileName + " for writing!\n";
        throw strErr;
    }

    map <string,string>::iterator iter = m_map.begin();
    while (iter != m_map.end()){
        file << iter->first << "\t" << iter->second << endl;
        iter++;
    }
    file.close();
}

void PassFile::AddUser(const char *user, const char *pass){
    sha2 mySha;
    m_boolChangesMade = true;
    string dat2hash = m_strSalt + user + (string) pass;
    m_map[user] = mySha.GetHash(m_ShaType, (const sha_byte *)dat2hash.c_str(), dat2hash.size());
}

bool PassFile::ChangePass(const char *user, const char *pass){
    if (m_map.find(user) == m_map.end()){
        return false;
    }
    AddUser(user, pass);
    return true;
}

bool PassFile::CheckUser(const char *user, const char *pass){
    sha2 mySha;
    string dat2hash = m_strSalt + user + (string) pass;
    string myHash = mySha.GetHash(m_ShaType, (const sha_byte *)dat2hash.c_str(), dat2hash.size());

    if (m_map.find(user) == m_map.end()){
        return false;
    }
    if (m_map[user] == myHash)
        return true;
    return false;
}


